#!/usr/bin/python3


##
# Prerequisites:
# A Touchscreen properly installed on your system:
# - a device to output to it, e.g. /dev/fb1
# - a device to get input from it, e.g. /dev/input/touchscreen
##

import pygame, time, evdev, select, math
import subprocess
import os
import netifaces
import multiprocessing 

#Colours
WHITE = (255,255,255)
BLACK = (0,0,0)
RED = (255,0,0)
GREEN = (0,255,0)
BLUE = (0,0,255)


#interface status
eth0_ok=0
wlan0_ok=0
wlan1_ok=0
wlan2_ok=0


mouse=[0,0]

def write_log(str):
    global log

    for i in range (LOGSIZE-1, 0,-1):
       lcd.blit(defaultFont.render(log[i], False, BLACK),(0, 300-(i*12)))
       log[i]=log[i-1] 
       lcd.blit(defaultFont.render(log[i], False, WHITE),(0, 300-(i*12)))
    lcd.blit(defaultFont.render(log[0], False, BLACK),(0, 300))
    log[0]=str
    lcd.blit(defaultFont.render(log[0], False, WHITE),(0, 300))


    refresh()
    time.sleep(0.1)



# Here we convert the evdev "hardware" touch coordinates into pygame surface pixel coordinates
def getPixelsFromCoordinates(coords):
    # TODO check divide by 0!
    if tftDelta [0] < 0:
        x = float(tftAbsDelta [0] - coords [0] + tftEnd [0]) / float(tftAbsDelta [0]) * float(surfaceSize [0])
    else:
        x = float(coords [0] - tftOrig [0]) / float(tftAbsDelta [0]) * float(surfaceSize [0])
    if tftDelta [1] < 0:
        y = float(tftAbsDelta [1] - coords [1] + tftEnd [1]) / float(tftAbsDelta [1]) * float(surfaceSize [1])
    else:
        y = float(coords [1] - tftOrig [1]) / float(tftAbsDelta [1]) * float(surfaceSize [1])
    #return (int(x), int(y))
    return [int(x), int(y)]

# Was useful to see what pieces I would need from the evdev events
def printEvent(event):
    print(evdev.categorize(event))
    print("Value: {0}".format(event.value))
    print("Type: {0}".format(event.type))
    print("Code: {0}".format(event.code))

def text_objects(text, font,color=WHITE):
    textSurface = font.render(text, True, color)
    return textSurface, textSurface.get_rect()

# This is the important bit
def refresh():
    # We open the TFT screen's framebuffer as a binary file. Note that we will write bytes into it, hence the "wb" operator
    f = open("/dev/fb0","wb")
    # According to the TFT screen specs, it supports only 16bits pixels depth
    # Pygame surfaces use 24bits pixels depth by default, but the surface itself provides a very handy method to convert it.
    # once converted, we write the full byte buffer of the pygame surface into the TFT screen framebuffer like we would in a plain file:
    f.write(lcd.convert(16,0).get_buffer())
    # We can then close our access to the framebuffer
    f.close()
    #time.sleep(0.1)


def button(lcd,msg,x,y,w,h,ic,ac,mouse,enable,action=None,next_layout=None):
   if enable:
       if x+w > mouse[0] > x and y+h > mouse[1] > y:
          pygame.draw.rect(lcd, ic,(x,y,w,h))
          text = pygame.font.SysFont("comicsansms",25)
          textSurf, textRect = text_objects(msg, text,ac)
          textRect.center = ( (x+(w/2)), (y+(h/2)) )
          lcd.blit(textSurf, textRect)
          refresh()
          time.sleep(0.1)

          #scroll_up()
          lcd.fill(BLACK)
          refresh()
          if action != None:
             action()
          global layout
          layout=next_layout
          layout(lcd,mouse)
  
          return 0
       else:
          pygame.draw.rect(lcd, ic,(x,y,w,h),1)
          text = pygame.font.SysFont("comicsansms",25)
          textSurf, textRect = text_objects(msg, text,ic)
          textRect.center = ( (x+(w/2)), (y+(h/2)) )
          lcd.blit(textSurf, textRect)
          return 1
   else:
      pygame.draw.rect(lcd, BLACK,(x,y,w,h))
      refresh()
      return 1

def main_layout(lcd,mouse):
    global eth0_ok
    global wlan0_ok
    global wlan1_ok
    global wlan2_ok

    button(lcd,"Copy Fusion to BACKUP",5,45,230,50,WHITE,BLACK,mouse,wlan1_ok,wireless_copy_files_to_backup.run,log_layout) and \
    button(lcd,"Copy Fusion to NFS",   5,100,230,50,WHITE,BLACK,mouse,wlan1_ok and (wlan2_ok or eth0_ok),wireless_copy_files_to_nfs.run,log_layout) and \
    button(lcd,"Network Info.",        5,155,230,50,WHITE,BLACK,mouse,1,show_network_info,log_layout) and \
    button(lcd,"Shutdown...",          5,210,230,50,WHITE,BLACK,mouse,1,None,dialog_shutdown_layout) and \
    button(lcd,"Reboot...",            240,210,230,50,WHITE,BLACK,mouse,1,None,dialog_reboot_layout)
   
def dialog_shutdown_layout(lcd,mouse):
    lcd.blit(defaultFont.render("Shutdown?", False, WHITE),(190, 70))
    button(lcd,"Yes",65,145,140,50,WHITE,BLACK,mouse,1,shutdown,main_layout) and \
    button(lcd,"No",265,145,140,50,WHITE,BLACK,mouse,1,None,main_layout) 

def dialog_reboot_layout(lcd,mouse):
    lcd.blit(defaultFont.render("Reboot?", False, WHITE),(190, 70))
    button(lcd,"Yes",65,145,140,50,WHITE,BLACK,mouse,1,reboot,main_layout) and \
    button(lcd,"No",265,145,140,50,WHITE,BLACK,mouse,1,None,main_layout) 
 
def scroll_up():
    for i in range(0,surfaceSize[1],1):
       pygame.draw.rect(lcd, BLACK,(0,surfaceSize[1]-i,surfaceSize[0],i))
       refresh()
    
    lcd.fill(BLACK)
    refresh()

 
def reboot():
    os.system('shutdown -r') 
    exit()

def shutdown():
    os.system('shutdown -P') 
    exit()

def show_network_info():
     reset_log_buffer()
     write_log("SSID: WAPPI")
     try:
        write_log("eth0 (NIC) : " + netifaces.ifaddresses('eth0')[netifaces.AF_INET][0]['addr']) 
     except:
        write_log("eth0 (NIC) : No IP Address") 
 
     try:
        write_log("wlan0 (internal wireless): " + netifaces.ifaddresses('wlan0')[netifaces.AF_INET][0]['addr']) 
     except:
        write_log("wlan0 (internal wireless): No IP Address") 
     
     try:
        write_log("wlan1 (5G USB): " + netifaces.ifaddresses('wlan1')[netifaces.AF_INET][0]['addr']) 
     except: 
        write_log("wlan1 (5G USB): No IP Adress") 

     try:
        write_log("wlan2 (TPLINK 3G USB): " + netifaces.ifaddresses('wlan2')[netifaces.AF_INET][0]['addr']) 
     except: 
        write_log("wlan2 (TPLINK 3G USB): No IP Address") 


def get_status():
     global eth0_ok
     global wlan0_ok
     global wlan1_ok
     global wlan2_ok


     try:
        netifaces.ifaddresses('eth0')[netifaces.AF_INET][0]['addr'] 
        eth0_ok=1
     except:
        eth0_ok=0
 
     try:
        netifaces.ifaddresses('wlan0')[netifaces.AF_INET][0]['addr'] 
        wlan0_ok=1
     except:
        wlan0_ok=0
    

     wlan1_ok=not os.system("nc -z -n -w 1 10.5.5.9 80") 
     #print(aa)
     #try:
     #   netifaces.ifaddresses('wlan1')[netifaces.AF_INET][0]['addr']
     #   wlan1_ok=1
     #except: 
     #   wlan1_ok=0

     try:
        netifaces.ifaddresses('wlan2')[netifaces.AF_INET][0]['addr'] 
        wlan2_ok=1
     except: 
        wlan2_ok=0



def log_layout(lcd,mouse):
    lcd.blit(defaultFont.render("Log:", False, WHITE),(0, 0))
    button(lcd,"Done",340,0,100,50,WHITE,BLACK,mouse,1,None,main_layout)



def open_log(filename):
    file_ptr=open(filename,"r")
    while True:
        line=file_ptr.readline()
        if line!="":
           print (file_ptr.readline())
        else:
           break
    return file_ptr



def read_log(file_ptr):
    if file_ptr != None:
       return file_ptr.readline()
    

def reset_log_buffer():
   global log
   log = [""]*LOGSIZE


def service_touchscreen(coor):
   
   X=0
   Y=0
   
   while True:
      # TODO get the right ecodes instead of int
      r,w,x = select.select([touch], [], [])
        
      
      for event in touch.read():
         #printEvent(event)
         if event.type == evdev.ecodes.EV_ABS:
            if event.code == 1:
               X = event.value
            elif event.code == 0:
               Y = event.value
            elif event.code == 24:
               P = event.value
      
         #elif event.type == evdev.ecodes.EV_KEY:
         if event.type == evdev.ecodes.EV_KEY:
            if event.code == 330 and event.value == 0:
               p = getPixelsFromCoordinates((X, Y))
               coor.put_nowait([p[0], p[1]])
               #oy=p[1]
               #print ("service" + str(ox) +" "+ str(oy))
               #w_pipe.write(str(p[0])  +" "+ str(p[1]))
               #w_pipe.flush()
                       
#so you can run this systemD
import signal
def handler(signum, frame):
    pass
   
#------------------------------------------------------------------------------------------------------------------------

if __name__== "__main__":

   try:
       signal.signal(signal.SIGHUP, handler)
   except AttributeError:
       # Windows compatibility
       pass
   
   
   
   LOGSIZE=25 
   log = [""]*LOGSIZE
   
   
   
   import  wireless_copy_files_to_backup
   import  wireless_copy_files_to_nfs
   
   
   
   # Very important: the exact pixel size of the TFT screen must be known so we can build graphics at this exact format
   surfaceSize = (480,320)
   
   # Note that we don't instantiate any display!
   pygame.init()
   
   lcd = pygame.Surface(surfaceSize)
   
   
   # Now we've got a function that can get the bytes from a pygame surface to the TFT framebuffer,
   # we can use the usual pygame primitives to draw on our surface before calling the refresh function.
   
   pygame.font.init()
   defaultFont = pygame.font.SysFont(None,20)
   
   # Used to map touch event from the screen hardware to the pygame surface pixels.
   # (Those values have been found empirically, use evtest to get numbers
   #tftOrig = (3750, 0)
   tftOrig = (3901, 295)
   #tftEnd = (150, 3750)
   tftEnd = (246,3775)
   tftDelta = (tftEnd [0] - tftOrig [0], tftEnd [1] - tftOrig [1])
   tftAbsDelta = (abs(tftEnd [0] - tftOrig [0]), abs(tftEnd [1] - tftOrig [1]))
   
   
   lcd.fill(BLACK)
   font_big = pygame.font.Font(None, 50)
   refresh()
   
   # We use evdev to read events from our touchscreen
   # (The device must exist and be properly installed for this to work)
   touch = evdev.InputDevice('/dev/input/event0')
   
   # We make sure the events from the touchscreen will be handled only by this program
   # (so the mouse pointer won't move on X when we touch the TFT screen)
   touch.grab()
   # Prints some info on how evdev sees our input device
   print(touch)
   # Even more info for curious people
   # print(touch.capabilities())
   
   get_status()
   
   layout=main_layout
   layout(lcd,(0,0))
   
   
   wireless_copy_files_to_backup.set_log_method(write_log)
   wireless_copy_files_to_nfs.set_log_method(write_log)
   refresh()
   

   #run service_touchscreen in a queue process
   coor=multiprocessing.Queue()
   p=multiprocessing.Process(target=service_touchscreen, args=(coor,))
   
   p.start()
   
   while True:
      #print (coor[:])
      if (coor.empty()):
         get_status()
         #print ("wlan1 "+str(wlan1_ok))
         layout(lcd,mouse)
         refresh()
      else:
         mouse=coor.get()
         layout(lcd,mouse)
         refresh()

   p.join()
   exit()
   
