#!/usr/bin/python3

from goprocam import GoProCamera
from goprocam import constants


import os
import sys
import re
import time
import netifaces

BACKUPPATH="/backup"
method_ptr=None


def set_log_method(method):
   global method_ptr
   method_ptr=method
   print("wireless_copy_giles_to_backup.set_log_method()")

def msg(str):
   global method_ptr
   if method_ptr!=None:
      method_ptr(str)
   else:
      print(str)

def camera_info(cam_ptr):
   msg("current mode: " + "" + cam_ptr.parse_value("mode", cam_ptr.getStatus(constants.Status.Status, constants.Status.STATUS.Mode)))
   msg("current submode: " + "" + cam_ptr.parse_value("sub_mode", cam_ptr.getStatus(constants.Status.Status, constants.Status.STATUS.SubMode)))
   msg("current video resolution: " + "" + cam_ptr.parse_value("video_res", cam_ptr.getStatus(constants.Status.Settings, constants.Video.RESOLUTION)))
   msg("current video framerate: " + "" + cam_ptr.parse_value("video_fr", cam_ptr.getStatus(constants.Status.Settings, constants.Video.FRAME_RATE)))
   msg("pictures taken: " + "" + str(cam_ptr.getStatus(constants.Status.Status, constants.Status.STATUS.PhotosTaken)))
   msg("videos taken: " + "" + str(cam_ptr.getStatus(constants.Status.Status, constants.Status.STATUS.VideosTaken)))
   msg("videos left: " + "" + cam_ptr.parse_value("video_left", cam_ptr.getStatus(constants.Status.Status, constants.Status.STATUS.RemVideoTime)))
   msg("pictures left: " + "" + str(cam_ptr.getStatus(constants.Status.Status, constants.Status.STATUS.RemPhotos)))
   msg("battery left: " + "" + cam_ptr.parse_value("battery", cam_ptr.getStatus(constants.Status.Status, constants.Status.STATUS.BatteryLevel)))
   msg("space left in sd card: " + "" + cam_ptr.parse_value("rem_space", cam_ptr.getStatus(constants.Status.Status, constants.Status.STATUS.RemainingSpace)))
   msg("camera SSID: " + "" + str(cam_ptr.getStatus(constants.Status.Status, constants.Status.STATUS.CamName)))
   msg("Is Recording: " + "" + cam_ptr.parse_value("recording", cam_ptr.getStatus(constants.Status.Status, constants.Status.STATUS.IsRecording)))
   msg("Clients connected: " + "" + str(cam_ptr.getStatus( constants.Status.Status, constants.Status.STATUS.IsConnected)))
   msg("camera model: " + "" + cam_ptr.infoCamera(constants.Camera.Name)) 
   msg("firmware version: " + "" + cam_ptr.infoCamera(constants.Camera.Firmware)) 
   msg("serial number: " + "" + cam_ptr.infoCamera(constants.Camera.SerialNumber)) 



def run(folder_dst=None,ptr=None):
 
   #global BACKUPPATH   
   if (ptr):
      log_ptr=ptr
 
   if folder_dst==None:
      msg ("Generating generic folder...")
      for i in range (0,1000):
         folder_dst=BACKUPPATH+"/CP_"+str(i)
         #if not(os.path.isdir(BACKUPPATH+"/CP_"+str(i))) or  not(os.path.exists(BACKUPPATH+"/CP_"+str(i))):
         if not(os.path.isdir(folder_dst)) or  not(os.path.exists(folder_dst)):
            break
      if i==1000:
         msg ("Can not generate CP_[0:1000]")
         return
      else:
         msg ("Done")
       
   elif os.path.isdir(folder_dst):
      msg ("directroy exist "+folder_dst+" exiting..")
      return
   elif os.path.exists(folder_dst):
      msg ("file exist "+folder_dst+" exiting..")
      return
   
   msg ("mkdir "+folder_dst)
   #os.system("mkdir "+folder_dst)
   #if not os.path.exists(folder_dst):
   os.makedirs(folder_dst);
   
   msg("Checking wlan1...")  
   try:
      msg(netifaces.ifaddresses('wlan1')[netifaces.AF_INET][0]['addr'])
   except:
      msg ("wlan1 not connected to Gopro SSID. exiting...")
      return
   
   
   gpCam = GoProCamera.GoPro()
   gpCam.pair(usepin=False)
   #gpCam.pair(usepin=True)
   
   
   
   msg ("Start...")
   
   localtime = time.asctime( time.localtime(time.time()) )
   start_time = time.time()
   msg("Start time :" +str(localtime))
   camera_info(gpCam)
   
   #for i in range(len(output)):
   #   msg(output[i])
   #msg(gpCam.getMedia())
   #msg(gpCam.getMediaInfo("file"))
   output=gpCam.listMedia(True,True)
   for i in range (len(output)):
      msg(str(output[i]))
   #exit()
   
   data=gpCam.listMedia(True,True)
   for i in data:
      folder=i[0]
      if not os.path.exists(folder_dst+"/"+folder):
         os.makedirs(folder_dst+"/"+folder);
      mp4_file=i[1]
      lrv_file=re.sub(".MP4",".LRV",mp4_file) 
    
      msg("copy "+folder+"/"+mp4_file+" to "+folder_dst); 
      gpCam.downloadMedia(folder,mp4_file,folder_dst+"/"+folder+"/"+mp4_file)
      msg("copy "+folder+"/"+lrv_file+" to "+folder_dst); 
      gpCam.downloadMedia(folder,lrv_file,folder_dst+"/"+folder+"/"+lrv_file)
      if re.search("GPBK",mp4_file) or  re.search("GB",mp4_file):
         wav_file=re.sub(".MP4",".WAV",mp4_file)
         #msg("copy "+folder+"/"+wav_file+" to "+folder_dst); 
         gpCam.downloadMedia(folder,wav_file,folder_dst+"/"+folder+"/"+wav_file)
      if re.search("GPBK",mp4_file):
         thm_file=re.sub(".MP4",".THM",mp4_file) 
         #msg("copy "+folder+"/"+thm_file+" to "+folder_dst); 
         gpCam.downloadMedia(folder,thm_file,folder_dst+"/"+folder+"/"+thm_file)
      if re.search("GPFR",mp4_file):
         thm_file=re.sub(".MP4",".THM",mp4_file) 
         #msg("copy "+folder+"/"+thm_file+" to "+folder_dst); 
         gpCam.downloadMedia(folder,thm_file,folder_dst+"/"+folder+"/"+thm_file)
         
   
   
   msg ("Done")
   localtime = time.asctime( time.localtime(time.time()) )
   elapsed_time = time.time() - start_time
   msg ("End time :", localtime)
   
   
   hours, rem = divmod(elasped_time, 3600)
   minutes, seconds = divmod(rem, 60)
   msg("Elaspse time: "+"{:0>2}:{:0>2}:{:05.2f}".format(int(hours),int(minutes),seconds))
   

if __name__ == '__main__':
   if len(sys.argv) < 2:
      msg ("Usage wireless_copy_files_to_nfs.py <destination directory> ")
      exit()
   
   run(sys.argv[1])
    
